------------------------------------------------------------------------------
-- $Id: rx_ll_if_sm.vhd,v 1.1 2008/02/08 04:15:10 shurt Exp $
------------------------------------------------------------------------------
-- rx_ll_if_sm.vhd            
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        rx_ll_if_sm.vhd
-- Version:         v1.00a
-- Description:     Controls data flow from the receive status fifo and hard
--                  fifo to LL
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the 
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DCW      2006.05.15      -- First version
--
--  <initials>      <date> 
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity rx_ll_if_sm is
    generic(
         C_RESET_ACTIVE  : std_logic := '1'
         );

    port(
         LocalLinkPaused         : out std_logic;
         PauseLocalLinkFull      : in  std_logic;
         FifoFullReset           : in  std_logic;
         ResetFifosOutOfSync     : out std_logic;

         SM_encoded              : out std_logic_vector(0 to 2);
         LLTemac_Clk             : in  std_logic;
         LLTemac_Rst             : in  std_logic;
         
         Footer_Done             : in  std_logic;
         Dump_Bad_Pckt           : out std_logic;
         
         RXFIFO2LL_Empty         : in  std_logic;
         RXFIFO2LL_Und           : in  std_logic;
--         RXFIFO2LL_Data          : in  std_logic_vector(0 to 35);
         RXFIFO2LL_Eop           : in  std_logic;
         LL2RXFIFO_Rden          : out std_logic;
         
         RSFIFO2LL_Empty         : in  std_logic;
         RSFIFO2LL_Und           : in  std_logic;
         RSFIFO2LL_Pckt_Status   : in  std_logic_vector(0 to 5);
         LL2RSFIFO_Rden          : out std_logic;
         
         TemacLL_DST_RDY_n       : in  std_logic;
         TemacLL_SOF_n           : out std_logic;             
         TemacLL_SOP_n           : out std_logic;
         TemacLL_EOF_n           : out std_logic;
         TemacLL_EOP_n           : out std_logic;             
         TemacLL_SRC_RDY_n       : out std_logic
         );
            
end rx_ll_if_sm;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

architecture beh of rx_ll_if_sm is

------------------------------------------------------------------------------
-- Type Declarations
------------------------------------------------------------------------------

type RX_SM_TYPE is (
                    OUT_OF_SYNC_WORK_AROUND,
                    IDLE,
                    RD_RXFIFO_ONCE,
                    RD_RSFIFO,
                    CHECK_STATUS,
                    SOF,
                    SOP,
                    RD_RXFIFO,
--                    EOP,
                    EOF,
                    DUMP_PCKT
                   );
                   
                   
------------------------------------------------------------------------------
-- Constant Declarations
------------------------------------------------------------------------------                   

constant PASS : std_logic_vector(0 to 5) := "111111";
constant FAIL : std_logic_vector(0 to 5) := "000000";

------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------

signal rx_sm_ps            : RX_SM_TYPE;
signal rx_sm_ns            : RX_SM_TYPE;
signal rx_pckt_status      : std_logic_vector(0 to 5);
signal rx_eop              : std_logic;


begin


------------------------------------------------------------------------------
-- Concurrent Assignments
------------------------------------------------------------------------------


rx_pckt_status <= RSFIFO2LL_Pckt_Status;
rx_eop <= RXFIFO2LL_Eop;

------------------------------------------------------------------------------
-- RX State Machine
-- RX_SM_SYNC_PROCESS: synchronous process of the state machine
-- RX_SM_CMB_PROCESS:  combinatorial next-state logic
------------------------------------------------------------------------------
-- This state machine determines the current state of the Transmitter
------------------------------------------------------------------------------

RX_SM_SYNC_PROCESS : process ( LLTemac_Clk )
begin
    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
        if (LLTemac_Rst = C_RESET_ACTIVE) then
            rx_sm_ps <= IDLE;
        else
            rx_sm_ps <= rx_sm_ns;
        end if;
    end if;
end process;

RX_SM_CMB_PROCESS : process (
                             rx_sm_ps,
                             RXFIFO2LL_Empty,
                             RXFIFO2LL_Und,
                             RSFIFO2LL_Empty,                             
                             RSFIFO2LL_Und,
                             TemacLL_DST_RDY_n,
                             Footer_Done,
                             rx_pckt_status,
                             rx_eop,
                             PauseLocalLinkFull
                           )
begin

   TemacLL_SOF_n <= '1';
   TemacLL_SOP_n <= '1';
   TemacLL_EOF_n <= '1';
   TemacLL_EOP_n <= '1';
   TemacLL_SRC_RDY_n <= '1';
   LL2RSFIFO_Rden <= '0';
   LL2RXFIFO_Rden <= '0';
   Dump_Bad_Pckt <= '0';
   ResetFifosOutOfSync <= '0';
   LocalLinkPaused <= '0';
    case rx_sm_ps is

        when IDLE =>        
            SM_encoded <= "000";
            if(PauseLocalLinkFull = '1') then
              rx_sm_ns <= IDLE;
              LocalLinkPaused <= '1';
            elsif((RSFIFO2LL_Empty='0') and (RXFIFO2LL_Empty='1')) then -- FIFOs are out of sync
               rx_sm_ns <= OUT_OF_SYNC_WORK_AROUND;            
            elsif((RSFIFO2LL_Empty='0') and (RXFIFO2LL_Empty='0')) then
               LL2RXFIFO_Rden <= '1';
               rx_sm_ns <= RD_RSFIFO;
            else
               rx_sm_ns <= IDLE;
            end if;
            
        when OUT_OF_SYNC_WORK_AROUND =>        
            SM_encoded <= "111";
            ResetFifosOutOfSync <= '1';
            rx_sm_ns <= IDLE;
           
            
        when RD_RSFIFO =>
            SM_encoded <= "001";
            if(RSFIFO2LL_Empty='0')then
               LL2RSFIFO_Rden <= '1';
               rx_sm_ns <= CHECK_STATUS;
            else
              if(RXFIFO2LL_Empty='0')then
                rx_sm_ns <= RD_RSFIFO;
              else
                rx_sm_ns <= IDLE;
              end if;
            end if;
            
        when CHECK_STATUS =>
            SM_encoded <= "010";
            if(rx_pckt_status=PASS) then
               rx_sm_ns <= SOF;
            elsif(rx_pckt_status=FAIL)then
               rx_sm_ns <= DUMP_PCKT;
            else
               rx_sm_ns <= OUT_OF_SYNC_WORK_AROUND;
            end if;
               
               
        when SOF =>
            SM_encoded <= "011";
            TemacLL_SRC_RDY_n <= '0';        
            if((TemacLL_DST_RDY_n='0') and
               (RXFIFO2LL_Empty <= '0')) then
               rx_sm_ns <= SOP;
               TemacLL_SOF_n <= '0';
            else
               rx_sm_ns <= SOF;
            end if;         


        when SOP =>
            SM_encoded <= "100";
            TemacLL_SRC_RDY_n <= '0';        
            if((TemacLL_DST_RDY_n='0') and
               (RXFIFO2LL_Empty='0')) then
               rx_sm_ns <= RD_RXFIFO;
               TemacLL_SOP_n <= '0';
               LL2RXFIFO_Rden <= '1';
            else
               rx_sm_ns <= SOP;
            end if;
            
        when RD_RXFIFO =>
            SM_encoded <= "101";
            TemacLL_SRC_RDY_n <= '0';        
            if(TemacLL_DST_RDY_n='0') then            
               if(RXFIFO2LL_Empty='0') and 
                 (rx_eop='0') then
                  LL2RXFIFO_Rden <= '1';
                  rx_sm_ns <= RD_RXFIFO;
               elsif(rx_eop='1') then
                  TemacLL_EOP_n <= '0';
                  rx_sm_ns <= EOF;
               else
                  rx_sm_ns <= RD_RXFIFO;
               end if;
            else
               rx_sm_ns <= RD_RXFIFO;
            end if;
               

        when EOF =>
            SM_encoded <= "110";
            TemacLL_SRC_RDY_n <= '0';
            if(TemacLL_DST_RDY_n='0') then
               if(Footer_Done='1') then
                  rx_sm_ns <= IDLE;
                  TemacLL_EOF_n <= '0';
               else
                  rx_sm_ns <= EOF;
               end if;
            else
               rx_sm_ns <= EOF;
            end if;
                                                                
                                          
        when DUMP_PCKT =>
            SM_encoded <= "111";
            Dump_Bad_Pckt <= '1';
            if(RXFIFO2LL_Empty='0') then
               LL2RXFIFO_Rden <= '1';            
               if(rx_eop='1') then
                  rx_sm_ns <= IDLE;
               else
                  rx_sm_ns <= DUMP_PCKT;
               end if;
            else
--               rx_sm_ns <= DUMP_PCKT;
               rx_sm_ns <= IDLE;
            end if;


        when others   =>  -- default to IDLE
            SM_encoded <= "000";
            rx_sm_ns <= IDLE;

    end case;
end process;


end beh;
