------------------------------------------------------------------------------
-- $Id: rx_temac_if.vhd,v 1.3 2008/06/19 21:41:00 mwelter Exp $
------------------------------------------------------------------------------
-- rx.vhd
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        rx_temac_if.vhd
-- Version:         v1.00a
-- Description:     Receive interface between csum and tx fifos and hard Temac
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DRP      2006.05.03      -- First version
--
--  <initials>      <date>
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Libraries used;
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;


--library proc_common_v2_00_a;
--use proc_common_v2_00_a.proc_common_pkg.max2;

library xps_ll_temac_v1_01_b;
use xps_ll_temac_v1_01_b.all;

--library csum_v1_00_a;
--use csum_v1_00_a.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

entity rx_temac_if is
    generic (
             C_FAMILY        : string    := "virtex5";  
             C_RESET_ACTIVE  : std_logic := '1';
             C_BUS_DWIDTH    : integer   := 32;
             C_TEMAC_TYPE    : integer   :=    0;  
               -- 0 - Virtex 5 hard TEMAC (FX, LXT, SXT devices)                
               -- 1 - Virtex 4 hard TEMAC (FX)               
               -- 2 - Soft TEMAC         
             C_TEMAC_RXCSUM  : integer   :=  0;
             C_CLIENT_DWIDTH : integer   :=  8;
             C_RX_LEN_LENGTH : integer   := 14
            );

    port    (
             LocalLinkPaused          : in  std_logic;
             PauseLocalLinkFull       : out std_logic;
             FifoFullReset            : out std_logic;
             LLTemac_Clk              : in  std_logic;
             LLTemac_Rst              : in  std_logic;
             
             RSFIFO2IP_Wrack          : in  std_logic;
             RSFIFO2IP_Full           : in  std_logic;             
             RSFIFO2IP_Ovr            : in  std_logic;
             RSFIFO2LL_Und            : in  std_logic;
             IP2RSFIFO_Wreq           : out std_logic;
             IP2RSFIFO_Data           : out std_logic_vector(0 to 35);

             RXFIFO2IP_Wrack          : in  std_logic; 
             RXFIFO2IP_Almost_Full    : in  std_logic;             
             RXFIFO2IP_Full           : in  std_logic;
             RXFIFO2IP_Ovr            : in  std_logic;
             RXFIFO2LL_Und            : in  std_logic;
             IP2RXFIFO_Wreq           : out std_logic;             
             Ip2RXFIFO_Wreq_unqualified : out std_logic;
             IP2RXFIFO_Data           : out std_logic_vector(0 to 35);
             

             RegCR_BrdCast_Rej        : in  std_logic;
             RegCR_MulCast_Rej        : in  std_logic;
             
             Pckt_Ovr_Run             : out std_logic;
             Rx_cmplt                 : out std_logic;
             Rx_Pckt_Rej              : out std_logic;             

             Rx_Cl_Clk                : in  std_logic;
             EmacClientRxBadFrame     : in  std_logic;
             EmacClientRxd            : in  std_logic_vector(C_CLIENT_DWIDTH-1 downto 0);
             EmacClientRxdVld         : in  std_logic;
             EmacClientRxFrameDrop    : in  std_logic;
             EmacClientRxGoodFrame    : in  std_logic;
             EmacClientRxStats        : in  std_logic_vector(6 downto 0);
             SoftEmacClientRxStats    : in  std_logic_vector(27 downto 0);
             EmacClientRxStatsVld     : in  std_logic;
--             TakeRxStatsNow           : in  std_logic;
             EmacClientRxStats_i      : in  std_logic_vector(6 downto 0);

             ClFifoOverflow           : out std_logic
            );
end rx_temac_if;

------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------

architecture beh of rx_temac_if is

------------------------------------------------------------------------------
-- Constant Declarations
------------------------------------------------------------------------------

--------------------------------------------------------------------------
-- Type Declarations
--------------------------------------------------------------------------

type RXSTATSSM_TYPE is (
                        IDLE,
                        SAMPLE0TO6,
                        SAMPLE7TO13,
                        SAMPLE14TO20
                       );

------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------
signal rx_Pckt_Rej_ii         : std_logic;
signal rx_Pckt_Rej_i         : std_logic;
signal rx_Pckt_Rej_d1        : std_logic;
signal rx_Pckt_Rej_d2        : std_logic;
signal rx_Pckt_Rej_d3        : std_logic;
signal rx_Pckt_Rej_d4        : std_logic;
signal rx_Pckt_Rej_d5        : std_logic;
signal pckt_Ovr_Run_i        : std_logic;
signal pckt_Ovr_Run_d1       : std_logic;
signal pckt_Ovr_Run_d2       : std_logic;
signal pckt_Ovr_Run_d3       : std_logic;
signal pckt_Ovr_Run_d4       : std_logic;
signal pckt_Ovr_Run_d5       : std_logic;
signal rx_Cmplt_i            : std_logic;
signal rx_cmplt_ii           : std_logic;
signal rx_cmplt_d1           : std_logic;
signal rx_cmplt_d2           : std_logic;
signal rx_cmplt_d3           : std_logic;
signal rx_cmplt_d4           : std_logic;
signal rx_cmplt_d5           : std_logic;

signal rxStatsSm_Cs          : RXSTATSSM_TYPE;
signal rxStatsSm_Ns          : RXSTATSSM_TYPE;
signal emacClientRxStats_d   : std_logic_vector(26 downto 0);
signal emacClientRxStats_cmb : std_logic_vector(26 downto 0);
signal takeRxStatsNow        : std_logic;
signal llTakeRxStatsNow      : std_logic;
signal llTakeRxStatsNow_d1   : std_logic;
signal llTakeRxStatsNow_new      : std_logic;
signal llTakeRxStatsNow_d1_new   : std_logic;
signal llTakeRxStatsNow_d2_new   : std_logic;


signal rx2Cl_IF_RdEn         : std_logic;
signal rx_Byte_Cnt_Rst       : std_logic;
signal rx_Byte_Cnt           : std_logic_vector(0 to C_RX_LEN_LENGTH-1);
signal cl_IF2Rx_F_Mt         : std_logic;
signal cl_Fifo_Rd_Ack        : std_logic;
signal cl_IF2Rx_Data         : std_logic_vector(0 to (2*C_CLIENT_DWIDTH-1) );
signal cl_If2Rx_BadFrame     : std_logic;
signal cl_If2Rx_GoodFrame    : std_logic;
signal cl_If2Rx_FrameDrop    : std_logic;

signal cl_IF2RxD_Val_Ind     : std_logic_vector(0 to (2*(C_CLIENT_DWIDTH/8)-1) );
signal cl_If2Rx_End_Of_Pckt  : std_logic;
signal rx2Cl_IF_Rd_Frm_Stats : std_logic;
signal rx2Cl_IF_Rd_Frm_Inds  : std_logic;
signal cl_If2Rx_Stats        : std_logic_vector(26 downto 0);
signal cl_If2Rx_BrdCast      : std_logic;
signal cl_If2Rx_MulCast      : std_logic;
--signal rxD_Reg_CE            : std_logic;

signal rx_csum_done          : std_logic;
signal csum_eop              : std_logic;
signal csum_eop_i            : std_logic;

signal rx_eop                : std_logic;
signal rx_sop                : std_logic;
signal tied_low              : std_logic_vector(0 to 31) := (others=>'0');
signal iP2RXFIFO_data_i      : std_logic_vector(0 to 31);
signal iP2RXFIFO_data_2i     : std_logic_vector(0 to 31);

signal good_bad_n            : std_logic_vector(0 to 5);
signal good_bad_status_n     : std_logic_vector(0 to 5);

signal rx_csum_data          : std_logic_vector(0 to 15);
signal iP2RXFIFO_wreq_i      : std_logic;

signal sop_i                 : std_logic;
signal rx_sop_i              : std_logic;
signal last_word             : std_logic;

signal remainder             : integer range 0 to 3;
signal rem_enc               : std_logic_vector(0 to (2*C_CLIENT_DWIDTH/8)-1);

signal rx_pckt_valid_i       : std_logic;
signal rx_pckt_valid         : std_logic;

signal sM_encoded            : std_logic_vector(0 to 2);
signal iP2RSFIFO_Wreq_int    : std_logic;
signal iP2RXFIFO_Wreq_int    : std_logic;             
signal iP2RXFIFO_Data_int    : std_logic_vector(0 to 35);
signal iP2RSFIFO_Data_int    : std_logic_vector(0 to 35);

signal cl_IF2Rx_EndOfFrameMarker     : std_logic;
signal cl_IF2Rx_FrameIsCorruptMarker : std_logic;

signal cl_Fifo_Rd_Ack_qual : std_logic;
signal LLTemac_Rst_qual    : std_logic;
signal LLTemac_Rst_qual2   : std_logic;

signal abortWrite          :  std_logic;

signal fifoFullReset_i     : std_logic;
signal pauseLocalLinkFull_i: std_logic;

signal cl_If2Rx_StatsVld     : std_logic;
signal takeRxStatsNow_p2     : std_logic;
signal takeRxStatsNow_d1     : std_logic;
signal takeRxStatsNow_d2     : std_logic;
signal takeRxStatsNow_d3     : std_logic;
signal takeRxStatsNow_d4     : std_logic;
signal takeRxStatsNow_or     : std_logic;

begin

StatsQual : process(LLTemac_Clk,LLTemac_Rst_qual2,takeRxStatsNow_p2)
   begin
      if(LLTemac_Rst_qual2='1' or takeRxStatsNow_p2 = '1') then
        llTakeRxStatsNow    <= '1';
        llTakeRxStatsNow_d1 <= '0';
      elsif(rising_edge(LLTemac_Clk)) then
        llTakeRxStatsNow_d1 <= llTakeRxStatsNow;
        llTakeRxStatsNow    <= '0';
      end if;
   end process;

StatsQual2 : process(LLTemac_Clk,LLTemac_Rst_qual2)
   begin
      if(rising_edge(LLTemac_Clk)) then
        if(LLTemac_Rst_qual2='1') then
          llTakeRxStatsNow_new    <= '0';
          llTakeRxStatsNow_d1_new <= '0';
          llTakeRxStatsNow_d2_new <= '0';
        elsif(takeRxStatsNow_or = '1') then
          llTakeRxStatsNow_new <= '1';
          llTakeRxStatsNow_d1_new    <= llTakeRxStatsNow_new;
          llTakeRxStatsNow_d2_new    <= llTakeRxStatsNow_d1_new;
        else
          llTakeRxStatsNow_new <= '0';
          llTakeRxStatsNow_d1_new    <= llTakeRxStatsNow_new;
          llTakeRxStatsNow_d2_new    <= llTakeRxStatsNow_d1_new;
        end if;
      end if;
   end process;

FifoFullReset      <= fifoFullReset_i;
PauseLocalLinkFull <= pauseLocalLinkFull_i;
------------------------------------------------------------------------------
-- Concurrent Signal Assignments
------------------------------------------------------------------------------
--Rx_Pckt_Rej      <= '0';
--Rx_Cmplt         <= rx_Cmplt_i and good_bad_status_n(0);
--Pckt_Ovr_Run    <= pckt_Ovr_Run_i;

rx_Cmplt_ii        <= rx_Cmplt_i and good_bad_status_n(0);


STRETCH_INTRPT : process(LLTemac_Clk)
   begin
     if (LLTemac_Clk'event and LLTemac_Clk = '1') then
       if (LLTemac_Rst = C_RESET_ACTIVE) then
         rx_cmplt_d1     <= '0';
         rx_cmplt_d2     <= '0';
         rx_cmplt_d3     <= '0';
         rx_cmplt_d4     <= '0';
         rx_cmplt_d5     <= '0';
         rx_Pckt_Rej_d1  <= '0';
         rx_Pckt_Rej_d2  <= '0';
         rx_Pckt_Rej_d3  <= '0';
         rx_Pckt_Rej_d4  <= '0';
         rx_Pckt_Rej_d5  <= '0';
         pckt_Ovr_Run_d1 <= '0';
         pckt_Ovr_Run_d2 <= '0';
         pckt_Ovr_Run_d3 <= '0';
         pckt_Ovr_Run_d4 <= '0';
         pckt_Ovr_Run_d5 <= '0';
       else
         rx_cmplt_d1 <= rx_cmplt_ii;
         rx_cmplt_d2 <= rx_cmplt_d1;
         rx_cmplt_d3 <= rx_cmplt_d2;
         rx_cmplt_d4 <= rx_cmplt_d3;
         rx_cmplt_d5 <= rx_cmplt_d4;
         Rx_cmplt    <= (rx_cmplt_ii OR rx_cmplt_d1 OR rx_cmplt_d2 OR rx_cmplt_d3 OR rx_cmplt_d4 OR rx_cmplt_d5);
         rx_Pckt_Rej_d1 <= rx_Pckt_Rej_i;
         rx_Pckt_Rej_d2 <= rx_Pckt_Rej_d1;
         rx_Pckt_Rej_d3 <= rx_Pckt_Rej_d2;
         rx_Pckt_Rej_d4 <= rx_Pckt_Rej_d3;
         rx_Pckt_Rej_d5 <= rx_Pckt_Rej_d4;
         rx_Pckt_Rej_ii    <= (rx_Pckt_Rej_i OR rx_Pckt_Rej_d1 OR rx_Pckt_Rej_d2 OR rx_Pckt_Rej_d3 OR rx_Pckt_Rej_d4 OR rx_Pckt_Rej_d5);
         pckt_Ovr_Run_d1 <= pckt_Ovr_Run_i;
         pckt_Ovr_Run_d2 <= pckt_Ovr_Run_d1;
         pckt_Ovr_Run_d3 <= pckt_Ovr_Run_d2;
         pckt_Ovr_Run_d4 <= pckt_Ovr_Run_d3;
         pckt_Ovr_Run_d5 <= pckt_Ovr_Run_d4;
         pckt_Ovr_Run    <= (pckt_Ovr_Run_i OR pckt_Ovr_Run_d1 OR pckt_Ovr_Run_d2 OR pckt_Ovr_Run_d3 OR pckt_Ovr_Run_d4 OR pckt_Ovr_Run_d5);
       end if;
     end if;
   end process;
   
Rx_Pckt_Rej <= rx_Pckt_Rej_ii;

cl_If2Rx_BrdCast <= cl_If2Rx_Stats(3);
cl_If2Rx_MulCast <= cl_If2Rx_Stats(4);

--rem_enc <= rem_enc_2i;
Ip2RXFIFO_Wreq_unqualified <= iP2RXFIFO_wreq_i;

--iP2RXFIFO_wreq_int <= iP2RXFIFO_wreq_i and not (RXFIFO2IP_full or rxFIFO2IP_Almost_Full);
iP2RXFIFO_wreq_int <= (iP2RXFIFO_wreq_i and not (RXFIFO2IP_full));-- or abortWrite ;-- or rxFIFO2IP_Almost_Full);
IP2RXFIFO_wreq	   <= iP2RXFIFO_wreq_int;
iP2RSFIFO_data_int <= rx_csum_data&rx_Byte_Cnt&good_bad_status_n;
IP2RSFIFO_data	   <= iP2RSFIFO_data_int;
iP2RXFIFO_data_int <= iP2RXFIFO_data_2i&rx_sop&rx_eop&rem_enc;
IP2RXFIFO_data	   <= iP2RXFIFO_data_int;

------------------------------------------------------------------------------
--  Component Instantiations
------------------------------------------------------------------------------

I_RX_TEMAC_IF_SM : entity xps_ll_temac_v1_01_b.rx_temac_if_sm
   generic map(
      C_RESET_ACTIVE  => C_RESET_ACTIVE
      )                                                                            
   port map(                                                                       
      llTakeRxStatsNow_d1   => llTakeRxStatsNow_new,     -- in
      --llTakeRxStatsNow_d1   => llTakeRxStatsNow_d1,     -- in
      LocalLinkPaused       => LocalLinkPaused,         -- in
      PauseLocalLinkFull    => pauseLocalLinkFull_i,    -- out
      FifoFullReset         => fifoFullReset_i,         -- out
      AbortWrite            => abortWrite,              -- out
      SM_encoded            => sM_encoded,              -- out

      LLTemac_Clk           => LLTemac_Clk,             -- in                      
      LLTemac_Rst           => LLTemac_Rst,             -- in                      
                                                                                   
      RXFIFO2IP_Almost_Full => RXFIFO2IP_Almost_Full,   -- in                      
      RXFIFO2IP_Full        => RXFIFO2IP_Full,          -- in                      
      RXFIFO2IP_Ovr         => RXFIFO2IP_Ovr,           -- in                                           
      RXFIFO2LL_Und         => RXFIFO2LL_Und,           -- in                      
                                                                                   
      RSFIFO2IP_Full        => RSFIFO2IP_Full,          -- in                      
      RSFIFO2IP_Ovr         => RSFIFO2IP_Ovr,           -- in                      
      RSFIFO2LL_Und         => RSFIFO2LL_Und,           -- in                      
                                                                                   
      Rx_Csum_done          => rx_csum_done,            -- in                      
      Rx_Byte_Cnt_Rst       => rx_byte_cnt_rst,         -- out                     
                                                                                   
      Rx2Cl_IF_RdEn         => rx2Cl_IF_RdEn,           -- out                     
      Cl_IF2Rx_F_Mt         => cl_IF2Rx_F_Mt,           -- in                          
      Cl_If2Rx_End_Of_Pckt  => cl_If2Rx_End_Of_Pckt,    -- in                      
      Cl_IF2Rx_EndOfFrameMarker     => cl_IF2Rx_EndOfFrameMarker,     -- in
      Cl_IF2Rx_FrameIsCorruptMarker => cl_IF2Rx_FrameIsCorruptMarker, -- in

      Cl_If2Rx_BadFrame     => cl_If2Rx_BadFrame,       -- in  
      Cl_If2Rx_FrameDrop    => cl_If2Rx_FrameDrop,      -- in
      
      Cl_If2Rx_BrdCast      => cl_If2Rx_BrdCast,        -- in
      Cl_If2Rx_MulCast      => cl_If2Rx_MulCast,        -- in
      Cl_If2Rx_GoodFrame    => cl_If2Rx_GoodFrame,      -- in
                                                                                               
      RegCR_BrdCast_Rej     => RegCR_BrdCast_Rej,       -- in                                           
      RegCR_MulCast_Rej     => RegCR_MulCast_Rej,       -- in                      
      Rx_cmplt              => rx_Cmplt_i,              -- out
      Rx_Pckt_Rej           => rx_Pckt_Rej_i,           -- out
      Pckt_Ovr_Run          => pckt_Ovr_Run_i,          -- out                     
      Rx_Sop                => sop_i,                   -- out                                               
      Rx_Eop                => rx_eop,                  -- out                      
      Csum_Eop              => open,--csum_eop,                -- out 
            
      Rx2Cl_IF_Rd_Frm_Inds  => rx2Cl_IF_Rd_Frm_Inds,    -- out
      Rx2Cl_IF_Rd_Frm_Stats => rx2Cl_IF_Rd_Frm_Stats,   -- out

      Cl_If2Rx_StatsVld	    => cl_If2Rx_StatsVld,       -- in
                           
      Good_bad_n            => good_bad_n,              -- out
      Last_Word             => last_word                -- out
      );  
      
                                                                                   
I_RX_CL_IF : entity xps_ll_temac_v1_01_b.rx_cl_if                                   
   generic map(                                                                     
      C_FAMILY        => C_FAMILY,
      C_RESET_ACTIVE  => C_RESET_ACTIVE,                                      
      C_TEMAC_TYPE    => C_TEMAC_TYPE,                                      
      C_CLIENT_DWIDTH => C_CLIENT_DWIDTH                                                            
      )                                                                                    
                                                                                           
   port map(                                                                               
      takeRxStatsNow_p2        => takeRxStatsNow_p2,       -- out
      llTakeRxStatsNow_d1   => llTakeRxStatsNow_new,     -- in
      --llTakeRxStatsNow_d1   => llTakeRxStatsNow_d1,     -- in
      PauseLocalLinkFull       => pauseLocalLinkFull_i,      -- in
      FifoFullReset            => fifoFullReset_i,           -- in
      LLTemac_Clk              => LLTemac_Clk,               -- in                         
      LLTemac_Rst              => LLTemac_Rst,               -- in                         
                                                                                           
      RXFIFO2IP_Almost_Full    => RXFIFO2IP_Almost_Full,     -- in                      
      RXFIFO2IP_Full           => RXFIFO2IP_Full,            -- in                      
      Rx2Cl_IF_RdEn            => rx2Cl_IF_RdEn,             -- in                         
      Cl_IF2Rx_F_Mt            => cl_IF2Rx_F_Mt,             -- out
      Cl_IF2Rx_Data            => cl_IF2Rx_Data,             -- out
      Cl_IF2RxD_Val_Ind        => cl_IF2RxD_Val_Ind,         -- out
      Cl_IF2Rx_EndOfFrameMarker     => cl_IF2Rx_EndOfFrameMarker,     -- out
      Cl_IF2Rx_FrameIsCorruptMarker => cl_IF2Rx_FrameIsCorruptMarker, -- out
      Cl_Fifo_Rd_Ack           => cl_Fifo_Rd_Ack,            -- out
      Rx2Cl_IF_Rd_Frm_Inds     => rx2Cl_IF_Rd_Frm_Inds,      -- in
      Cl_If2Rx_End_Of_Pckt     => cl_If2Rx_End_Of_Pckt,      -- out
      Rx_Cmplt                 => rx_Cmplt_i,                -- in
      Cl_If2Rx_BadFrame        => cl_If2Rx_BadFrame,         -- out
      Cl_If2Rx_GoodFrame       => cl_If2Rx_GoodFrame,        -- out
      Cl_If2Rx_FrameDrop       => cl_If2Rx_FrameDrop,        -- out

      Rx2Cl_IF_Rd_Frm_Stats    => rx2Cl_IF_Rd_Frm_Stats,     -- in
      Cl_If2Rx_Stats           => cl_If2Rx_Stats,            -- out

      Rx_Cl_Clk                => Rx_Cl_Clk,                 -- in
      EmacClientRxBadFrame     => EmacClientRxBadFrame,      -- in
      EmacClientRxd            => EmacClientRxd,             -- in
      EmacClientRxdVld         => EmacClientRxdVld,          -- in
      EmacClientRxFrameDrop    => EmacClientRxFrameDrop,     -- in
      EmacClientRxGoodFrame    => EmacClientRxGoodFrame,     -- in
      EmacClientRxStats        => emacClientRxStats_d,       -- in
      EmacClientRxStatsVld     => EmacClientRxStatsVld,      -- in
      TakeRxStatsNow           => takeRxStatsNow,            -- in
      EmacClientRxStats_i      => EmacClientRxStats_i,       -- in
      SoftEmacClientRxStats    => SoftEmacClientRxStats,     -- in
      Cl_If2Rx_StatsVld	       => cl_If2Rx_StatsVld,         -- out

      ClFifoOverflow           => ClFifoOverflow             -- out
      );

cl_Fifo_Rd_Ack_qual <= cl_Fifo_Rd_Ack and not(rx_Pckt_Rej_ii);
LLTemac_Rst_qual    <= LLTemac_Rst or rx_Pckt_Rej_ii or fifoFullReset_i;
LLTemac_Rst_qual2   <= LLTemac_Rst or fifoFullReset_i;

I_RX_DATA_REG : entity xps_ll_temac_v1_01_b.rx_data_reg
   generic map(
      C_RESET_ACTIVE  => C_RESET_ACTIVE,
      C_CLIENT_DWIDTH => C_CLIENT_DWIDTH,
      C_BUS_DWIDTH    => C_BUS_DWIDTH
      )
   port map(
      LLTemac_Clk       => LLTemac_Clk,       -- in
      LLTemac_Rst       => LLTemac_Rst_qual,       -- in
      RxD_Reg_CE        => cl_Fifo_Rd_Ack_qual,    -- in
      Partial_Wr        => last_word,         -- in
      Rx_Cmplt          => rx_Cmplt_i,        -- in
      Cl_IF2RxD_Val_Ind => cl_IF2RxD_Val_Ind, -- in
      Cl_IF2Rx_Data     => cl_IF2Rx_Data,     -- in
      IP2RFIFO_Data     => iP2RXFIFO_data_i,  -- out
      IP2RFIFO_WrReq    => iP2RXFIFO_wreq_i   -- out
      );
      
      
I_RX_BYTE_CNTR : entity xps_ll_temac_v1_01_b.rx_byte_cntr
   generic map (
      C_RESET_ACTIVE  => C_RESET_ACTIVE,
      C_CLIENT_DWIDTH => C_CLIENT_DWIDTH,
      C_RX_LEN_LENGTH => C_RX_LEN_LENGTH
      )
   port map(
      LLTemac_Clk       => LLTemac_Clk,         -- in  
      LLTemac_Rst       => LLTemac_Rst_qual2,         -- in  
      Rx_Byte_Cnt_Rst   => rx_Byte_Cnt_Rst,     -- in  
      Cl_IF2RxD_Val_Ind => cl_IF2RxD_Val_Ind,   -- in  
      Cl_Fifo_Rd_Ack    => cl_Fifo_Rd_Ack,      -- in  
      No_Valid_data_n   => open,                -- out 
      Rx_Byte_Cnt       => rx_Byte_Cnt          -- out 
      );
      
      
GEN_RXCSUM : if(C_TEMAC_RXCSUM = 1) generate
   I_RX_CSUM : entity xps_ll_temac_v1_01_b.rx_csum_top 
      port map (
         clk      => LLTemac_Clk,         --in  
         rst      => LLTemac_Rst_qual2,         --in  
         err      => '0',                 --in  
         eop      => csum_eop,            --in  
         cscntrl  => '0',                 --in  
         data_in  => iP2RXFIFO_Data_2i,   --in  
         release  => rx_csum_done,        --out 
         wr_ack   => iP2RXFIFO_wreq_i,    --in  
         wr_req   => iP2RSFIFO_Wreq_int,      --out 
         wr_data  => rx_csum_data         --out 
         );
  IP2RSFIFO_Wreq  <= iP2RSFIFO_Wreq_int;
end generate;

GEN_NO_RXCSUM : if(C_TEMAC_RXCSUM = 0) generate
   
--   rx_csum_done <= '0';
   rx_csum_done <= '1';
   
   iP2RSFIFO_Wreq_int <= rx_eop;
   IP2RSFIFO_Wreq     <= iP2RSFIFO_Wreq_int;
   rx_csum_data <= (others=>'0');
end generate;
      
      
------------------------------------------------------------------------------
--  This process aligns the State Machine generated start-of-packet with
--  the data that is written to the RXFIFO
------------------------------------------------------------------------------



SOP_ALIGN : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst_qual2='1' or iP2RXFIFO_wreq_i='1') then
            rx_sop_i <= '0';
          elsif(sop_i='1') then
            rx_sop_i <= '1';
          end if;
      end if;
   end process;

rx_sop <= '1' when (rx_sop_i='1' and iP2RXFIFO_wreq_i='1')
              else '0';
      
      

------------------------------------------------------------------------------
--  This process creates the csum_eop signal
------------------------------------------------------------------------------

process(LLTemac_Clk)
begin
   if(rising_edge(LLTemac_Clk)) then
      if(LLTemac_Rst_qual2='1') then
         csum_eop <= '0';
         csum_eop_i <= '0';
      else
         csum_eop_i <= last_word;
         csum_eop <= csum_eop_i;
      end if;
   end if;
end process;
         
         
          
------------------------------------------------------------------------------
--  This process masks invalid remaining bytes and encodes the remainder
------------------------------------------------------------------------------

RX_VALID_PACKET : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst_qual2='1' or rx_csum_done='1') then
            rx_pckt_valid_i <= '0';
         elsif(rx_sop='1') then
            rx_pckt_valid_i <= '1';
         end if;
      end if;
   end process;
   
rx_pckt_valid <= rx_sop or rx_pckt_valid_i or rx_csum_done;

--remainder  <= to_integer((unsigned(rx_Byte_Cnt)) rem 4);

GET_REMAINDER: process (rx_Byte_Cnt) -- fix for problem with unsigned
begin
  case rx_Byte_Cnt(12 to 13) is 
    when "00" =>
      remainder <= 0;
    when "01" =>
      remainder <= 1; 
    when "10" =>
      remainder <= 2; 
    when "11" =>
      remainder <= 3; 
    when others  =>
      remainder <= 0;
  end case;
end process;

MASK_REM_ENC : process(LLTemac_Rst_qual2, rem_enc, rx_pckt_valid, remainder, 
                       iP2RXFIFO_Data_i, rx_eop)
   begin
      if(LLTemac_Rst_qual2='1') then
         rem_enc <= "00";
         iP2RXFIFO_Data_2i <= (others=>'0');
      elsif(rx_pckt_valid='1') then
         if(rx_eop='1') then
            case remainder is
               when 0 => 
                  rem_enc <= "00";
                  iP2RXFIFO_Data_2i <= iP2RXFIFO_Data_i;                  
               when 1 => 
                  rem_enc <= "01";
                  iP2RXFIFO_Data_2i <= iP2RXFIFO_Data_i(0 to 7)&tied_low(8 to 31);
            
               when 2 => 
                  rem_enc <= "10";
                  iP2RXFIFO_Data_2i <= iP2RXFIFO_Data_i(0 to 15)&tied_low(16 to 31);
               
               when 3 => 
                  rem_enc <= "11";
                  iP2RXFIFO_Data_2i <= iP2RXFIFO_Data_i(0 to 23)&tied_low(24 to 31);
               
               when others => 
                  null;
               
            end case;
         else
            rem_enc <= "00";
            iP2RXFIFO_Data_2i <= iP2RXFIFO_Data_i;
         end if;
      else
         iP2RXFIFO_Data_2i <= (others=>'0');
     end if;
   end process;
     
         
------------------------------------------------------------------------------
--  This process latches the bad packet status until written to status fifo
------------------------------------------------------------------------------

--BAD_PCKT_STATUS : process(LLTemac_Rst, good_bad_n, rx_sop)
--   begin
--      if(LLTemac_Rst='1' or (rx_sop='1' and rx_Pckt_Rej_ii='0')) then
--         good_bad_status_n <= (others=>'1');
--      
--      elsif(good_bad_n(0)='0') then
--         good_bad_status_n <= (others=>'0');
--      
--      end if;
--   end process;
         
  BAD_PCKT_STATUS : process(LLTemac_Rst_qual2, good_bad_n, rx_sop,LLTemac_Clk)
  begin
    if(rising_edge(LLTemac_Clk)) then
      if(LLTemac_Rst_qual2='1' or (rx_sop='1' and rx_Pckt_Rej_ii='0')) then
         good_bad_status_n <= (others=>'1');      
      elsif(good_bad_n(0)='0') then
         good_bad_status_n <= (others=>'0');
      else
         good_bad_status_n <= good_bad_status_n;
      end if;
    end if;
  end process;
         

      

  --------------------------------------------------------------------------
  -- Rx Statistics Capture State Machine
  -- RXSTATSSM_REGS_PROCESS: registered process of the state machine
  -- RXSTATSSM_CMB_PROCESS:  combinatorial next-state logic
  --------------------------------------------------------------------------
  
  RXSTATSSM_REGS_PROCESS: process ( LLTemac_Rst_qual2, Rx_Cl_Clk )
  begin
    if (LLTemac_Rst_qual2 = '1') then
      rxStatsSm_Cs        <= IDLE;
      emacClientRxStats_d <= (others => '0');
      takeRxStatsNow_d1 <= '0';
      takeRxStatsNow_d2 <= '0';
      takeRxStatsNow_d3 <= '0';
      takeRxStatsNow_d4 <= '0';
      takeRxStatsNow_or <= '0';
    elsif (rising_edge(Rx_Cl_Clk)) then
      rxStatsSm_Cs        <= rxStatsSm_Ns;
      emacClientRxStats_d <= emacClientRxStats_cmb; 
      takeRxStatsNow_d1 <= takeRxStatsNow;
      takeRxStatsNow_d2 <= takeRxStatsNow_d1;
      takeRxStatsNow_d3 <= takeRxStatsNow_d2;
      takeRxStatsNow_d4 <= takeRxStatsNow_d3;
      takeRxStatsNow_or <= takeRxStatsNow_d2 or takeRxStatsNow_d3;
    end if;
  end process;
  
  RXSTATSSM_CMB_PROCESS: process (
                                  RxStatsSm_Cs,
                                  EmacClientRxStats,
                                  emacClientRxStats_d,
                                  emacClientRxStatsVld
                                 )

  begin

    case rxStatsSm_Cs is

      when IDLE =>
          emacClientRxStats_cmb(26 downto 7) <= (others => '0');
          emacClientRxStats_cmb(6 downto 0)  <= EmacClientRxStats(6 downto 0);
          takeRxStatsNow                     <= '0';
          if (EmacClientRxStatsVld = '1') then
              rxStatsSm_Ns <= SAMPLE0TO6;
          else
              rxStatsSm_Ns <= IDLE;
          end if;

      when SAMPLE0TO6 =>
          emacClientRxStats_cmb(26 downto 14) <= (others => '0');
          emacClientRxStats_cmb(13 downto 7)  <= EmacClientRxStats(6 downto 0);
          emacClientRxStats_cmb(6 downto 0)   <= emacClientRxStats_d(6 downto 0);
          takeRxStatsNow                      <= '0';
          rxStatsSm_Ns <= SAMPLE7TO13;

      when SAMPLE7TO13 =>
          emacClientRxStats_cmb(26 downto 21) <= (others => '0');
          emacClientRxStats_cmb(20 downto 14) <= EmacClientRxStats(6 downto 0);
          emacClientRxStats_cmb(13 downto 0)  <= emacClientRxStats_d(13 downto 0);
          takeRxStatsNow                      <= '0';
          rxStatsSm_Ns <= SAMPLE14TO20;

      when SAMPLE14TO20 =>
          emacClientRxStats_cmb(26 downto 21) <= EmacClientRxStats(5 downto 0);
          emacClientRxStats_cmb(20 downto 0)  <= emacClientRxStats_d(20 downto 0);
          takeRxStatsNow                      <= '1';
          rxStatsSm_Ns <= IDLE;

    end case;
  end process;
  
end beh;
